import pytest
from pysnmp.hlapi.asyncio import *

@pytest.mark.asyncio
async def test_custom_asn1_mib_search_path():
    snmpEngine = SnmpEngine()
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        CommunityData('public'),
        UdpTransportTarget(('demo.pysnmp.com', 161)),
        ContextData(),
        ObjectType(ObjectIdentity('IF-MIB', 'ifInOctets', 1).addAsn1MibSource('file:///usr/share/snmp',
                                                                                'https://mibs.pysnmp.com/asn1/@mib@'))
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert len(varBinds) == 1
    assert varBinds[0][0].prettyPrint() == 'IF-MIB::ifInOctets.1'

    snmpEngine.transportDispatcher.closeDispatcher()


"""
SNMPv1
++++++

Send SNMP GET request using the following options:

  * with SNMPv1, community 'public'
  * over IPv4/UDP
  * to an Agent at demo.pysnmp.com:161
  * for an instance of SNMPv2-MIB::sysDescr.0 MIB object
  * Based on asyncio I/O framework

Functionally similar to:

| $ snmpgetnext -v1 -c public demo.pysnmp.com SNMPv2-MIB::sysDescr.0

"""#
import asyncio
from pysnmp.hlapi.asyncio.slim import Slim
from pysnmp.smi.rfc1902 import ObjectIdentity, ObjectType

async def run():
    slim = Slim(1)
    errorIndication, errorStatus, errorIndex, varBinds = await slim.next(
        'public',
        'demo.pysnmp.com',
        161,
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    if errorIndication:
        print(errorIndication)
    elif errorStatus:
        print(
            "{} at {}".format(
                errorStatus.prettyPrint(),
                errorIndex and varBinds[int(errorIndex) - 1][0] or "?",
            )
        )
    else:
        for varBind in varBinds:
            print(" = ".join([x.prettyPrint() for x in varBind]))

    slim.close()


asyncio.run(run())



import pytest
from pysnmp.smi import builder, view

mibBuilder = builder.MibBuilder()
mibBuilder.addMibSources(builder.DirMibSource("/opt/pysnmp_mibs"))
mibBuilder.loadModules(
    "SNMPv2-MIB", "SNMP-FRAMEWORK-MIB", "SNMP-COMMUNITY-MIB", "IP-MIB"
)
mibView = view.MibViewController(mibBuilder)

def test_getNodeName_by_OID():
    oid, label, suffix = mibView.getNodeName((1, 3, 6, 1, 2, 1, 1, 1))
    assert oid == (1, 3, 6, 1, 2, 1, 1, 1)
    assert label == ('iso', 'org', 'dod', 'internet', 'mgmt', 'mib-2', 'system', 'sysDescr')
    assert suffix == ()

def test_getNodeName_by_label():
    oid, label, suffix = mibView.getNodeName((1, 3, 6, 1, 2, "mib-2", 1, "sysDescr"))
    assert oid == (1, 3, 6, 1, 2, 1, 1, 1)
    assert label == ('iso', 'org', 'dod', 'internet', 'mgmt', 'mib-2', 'system', 'sysDescr')
    assert suffix == ()

def test_getNodeName_by_symbol_description():
    oid, label, suffix = mibView.getNodeName(("sysDescr",))
    assert oid == (1, 3, 6, 1, 2, 1, 1, 1)
    assert label == ('iso', 'org', 'dod', 'internet', 'mgmt', 'mib-2', 'system', 'sysDescr')
    assert suffix == ()

def test_getNodeName_by_symbol_description_with_module_name():
    oid, label, suffix = mibView.getNodeName(("snmpEngineID",), "SNMP-FRAMEWORK-MIB")
    assert oid == (1, 3, 6, 1, 6, 3, 10, 2, 1, 1)
    assert label == ('iso', 'org', 'dod', 'internet', 'snmpV2', 'snmpModules', 'snmpFrameworkMIB', 'snmpFrameworkMIBObjects', 'snmpEngine', 'snmpEngineID')
    assert suffix == ()

    (mibNode,) = mibBuilder.importSymbols("SNMP-FRAMEWORK-MIB", "snmpEngineID")
    assert mibNode.syntax.prettyPrint() != ''

def test_getNodeName_by_symbol_location_lookup_by_name():
    modName, symName, suffix = mibView.getNodeLocation(("snmpCommunityEntry",))
    assert modName == "SNMP-COMMUNITY-MIB"
    assert symName == "snmpCommunityEntry"
    assert suffix == ()


import pytest
from pysnmp.hlapi.asyncio import *
from pysnmp.proto.errind import DecryptionError, RequestTimedOut, UnknownUserName

@pytest.mark.asyncio
async def test_usm_sha_aes128():
    snmpEngine = SnmpEngine()
    authData = UsmUserData(
        "usr-sha-aes",
        "authkey1",
        "privkey1",
        authProtocol=usmHMACSHAAuthProtocol,
        privProtocol=usmAesCfb128Protocol,
    )
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        authData,
        UdpTransportTarget(("demo.pysnmp.com", 161), retries=0),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert len(varBinds) == 1
    assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
    isinstance(varBinds[0][1], OctetString)

    snmpEngine.transportDispatcher.closeDispatcher()

@pytest.mark.asyncio
async def test_usm_sha_aes128_wrong_auth():
    snmpEngine = SnmpEngine()
    authData = UsmUserData(
        "usr-sha-aes",
        "authkey1",
        "privkey1",
        authProtocol=usmHMACMD5AuthProtocol, # wrongly use usmHMACMD5AuthProtocol
        privProtocol=usmAesCfb128Protocol
    )
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        authData,
        UdpTransportTarget(("demo.pysnmp.com", 161), retries=0),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert isinstance(errorIndication, DecryptionError)
    assert str(errorIndication) == 'Ciphering services not available or ciphertext is broken'

    snmpEngine.transportDispatcher.closeDispatcher()

@pytest.mark.asyncio
async def test_usm_sha_aes128_wrong_priv():
    snmpEngine = SnmpEngine()
    authData = UsmUserData(
        "usr-sha-aes",
        "authkey1",
        "privkey1",
        authProtocol=usmHMACSHAAuthProtocol,
        privProtocol=usmDESPrivProtocol, # wrongly use usmDESPrivProtocol
    )
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        authData,
        UdpTransportTarget(("demo.pysnmp.com", 161), retries=0),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert isinstance(errorIndication, DecryptionError)
    assert str(errorIndication) == 'Ciphering services not available or ciphertext is broken'

    snmpEngine.transportDispatcher.closeDispatcher()

@pytest.mark.asyncio
async def test_usm_sha_aes128_wrong_user():
    snmpEngine = SnmpEngine()
    authData = UsmUserData(
        "usr-sha-aes-not-exist",
        "authkey1",
        "privkey1",
        authProtocol=usmHMACSHAAuthProtocol,
        privProtocol=usmAesCfb128Protocol,
    )
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        authData,
        UdpTransportTarget(("demo.pysnmp.com", 161), retries=0),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert isinstance(errorIndication, UnknownUserName)
    assert str(errorIndication) == 'Unknown USM user'

    snmpEngine.transportDispatcher.closeDispatcher()



"""
Multiple SNMP USM users
+++++++++++++++++++++++

Listen and respond to SNMP GET/SET/GETNEXT/GETBULK queries with
the following options:

* SNMPv3
* with USM user:
    - 'usr-md5-des', auth: MD5, priv DES or
    - 'usr-sha-none', auth: SHA, no privacy
    - 'usr-sha-aes128', auth: SHA, priv AES
* allow access to SNMPv2-MIB objects (1.3.6.1.2.1)
* over IPv4/UDP, listening at 127.0.0.1:161
* using asyncio network transport (available since Python 3.4)

Either of the following Net-SNMP commands will walk this Agent:

| $ snmpwalk -v3 -u usr-md5-des -l authPriv -A authkey1 -X privkey1 localhost .1.3.6
| $ snmpwalk -v3 -u usr-sha-none -l authNoPriv -a SHA -A authkey1 localhost .1.3.6
| $ snmpwalk -v3 -u usr-sha-aes128 -l authPriv -a SHA -A authkey1 -x AES -X privkey1 localhost .1.3.6

Requires Python 3.4 and later!

"""#
from pysnmp.entity import engine, config
from pysnmp.entity.rfc3413 import cmdrsp, context
from pysnmp.carrier.asyncio.dgram import udp
import asyncio

# Get the event loop for this thread
loop = asyncio.get_event_loop()

# Create SNMP engine with autogenernated engineID and pre-bound
# to socket transport dispatcher
snmpEngine = engine.SnmpEngine()

# Transport setup

# UDP over IPv4
config.addTransport(
    snmpEngine, udp.domainName, udp.UdpTransport().openServerMode(("127.0.0.1", 161))
)

# SNMPv3/USM setup

# user: usr-md5-des, auth: MD5, priv DES
config.addV3User(
    snmpEngine,
    "usr-md5-des",
    config.usmHMACMD5AuthProtocol,
    "authkey1",
    config.usmDESPrivProtocol,
    "privkey1",
)
# user: usr-sha-none, auth: SHA, priv NONE
config.addV3User(snmpEngine, "usr-sha-none", config.usmHMACSHAAuthProtocol, "authkey1")
# user: usr-sha-none, auth: SHA, priv AES
config.addV3User(
    snmpEngine,
    "usr-sha-aes128",
    config.usmHMACSHAAuthProtocol,
    "authkey1",
    config.usmAesCfb128Protocol,
    "privkey1",
)

# Allow full MIB access for each user at VACM
config.addVacmUser(
    snmpEngine, 3, "usr-md5-des", "authPriv", (1, 3, 6, 1, 2, 1), (1, 3, 6, 1, 2, 1)
)
config.addVacmUser(
    snmpEngine, 3, "usr-sha-none", "authNoPriv", (1, 3, 6, 1, 2, 1), (1, 3, 6, 1, 2, 1)
)
config.addVacmUser(
    snmpEngine, 3, "usr-sha-aes128", "authPriv", (1, 3, 6, 1, 2, 1), (1, 3, 6, 1, 2, 1)
)

# Get default SNMP context this SNMP engine serves
snmpContext = context.SnmpContext(snmpEngine)

# Register SNMP Applications at the SNMP engine for particular SNMP context
cmdrsp.GetCommandResponder(snmpEngine, snmpContext)
cmdrsp.SetCommandResponder(snmpEngine, snmpContext)
cmdrsp.NextCommandResponder(snmpEngine, snmpContext)
cmdrsp.BulkCommandResponder(snmpEngine, snmpContext)

# Run asyncio main loop
loop.run_forever()



"""
Agent operations on MIB
+++++++++++++++++++++++

This script explains how SNMP Agent application manipulates
its MIB possibly triggered by SNMP Manager's commands.

"""#
# SNMP agent backend e.g. Agent access to Managed Objects
from pysnmp.smi import builder, instrum, exval

print("Loading MIB modules..."),
mibBuilder = builder.MibBuilder().loadModules(
    "SNMPv2-MIB", "SNMP-FRAMEWORK-MIB", "SNMP-COMMUNITY-MIB"
)
print("done")

print("Building MIB tree..."),
mibInstrum = instrum.MibInstrumController(mibBuilder)
print("done")

print("Building table entry index from human-friendly representation..."),
(snmpCommunityEntry,) = mibBuilder.importSymbols(
    "SNMP-COMMUNITY-MIB", "snmpCommunityEntry"
)
instanceId = snmpCommunityEntry.getInstIdFromIndices("my-router")
print("done")

print("Create/update SNMP-COMMUNITY-MIB::snmpCommunityEntry table row: ")
varBinds = mibInstrum.writeVars(
    (
        (snmpCommunityEntry.name + (2,) + instanceId, "mycomm"),
        (snmpCommunityEntry.name + (3,) + instanceId, "mynmsname"),
        (snmpCommunityEntry.name + (7,) + instanceId, "volatile"),
    )
)
for oid, val in varBinds:
    print(
        "{} = {}".format(
            ".".join([str(x) for x in oid]),
            not val.isValue and "N/A" or val.prettyPrint(),
        )
    )
print("done")

print("Read whole MIB (table walk)")
oid, val = (), None
while True:
    oid, val = mibInstrum.readNextVars(((oid, val),))[0]
    if exval.endOfMib.isSameTypeWith(val):
        break
    print(
        "{} = {}".format(
            ".".join([str(x) for x in oid]),
            not val.isValue and "N/A" or val.prettyPrint(),
        )
    )
print("done")

print("Unloading MIB modules..."),
mibBuilder.unloadModules()
print("done")



from pysnmp.proto.rfc1902 import ObjectIdentifier
from pysnmp.smi import builder, view, compiler

# Create MIB builder
mibBuilder = builder.MibBuilder()

# Optionally compile MIBs
compiler.addMibCompiler(mibBuilder, sources=['/usr/share/snmp/mibs'])

mibBuilder.loadTexts = True

# Load MIB modules
mibBuilder.loadModules('SNMPv2-MIB')
# mibBuilder.addMibSources(builder.DirMibSource('/Users/lextm/pysnmp.com/pysnmp/mibs'))
# mibBuilder.loadModule('LEXTUDIO-MIB')

# Create MIB view controller
mibViewController = view.MibViewController(mibBuilder)

# Create an OID object
oid = ObjectIdentifier('1.3.6.1.2.1.1.3.0')

# Get the MIB name and symbol name for the OID
modName, symName, suffix = mibViewController.getNodeLocation(oid)

# Get the MIB node for the OID
mibNode, = mibBuilder.importSymbols(modName, symName)

# Get the description of the MIB node
description = mibNode.getDescription()

# Print the results
print('OID: %s' % oid)
print('MIB name: %s' % modName)
print('Symbol name: %s' % symName)
print('Description: %s' % description)



import pytest
from pysnmp.hlapi.asyncio.slim import Slim
from pysnmp.smi.rfc1902 import ObjectIdentity, ObjectType

@pytest.mark.asyncio
async def test_v2c_bulk():
    slim = Slim()
    errorIndication, errorStatus, errorIndex, varBinds = await slim.bulk(
        'public',
        'demo.pysnmp.com',
        161,
        0,
        50,
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert len(varBinds) > 0

    slim.close()


import pytest
from pysnmp.hlapi.asyncio import *

@pytest.mark.asyncio
async def test_send_notification():
    snmpEngine = SnmpEngine()
    errorIndication, errorStatus, errorIndex, varBinds = await sendNotification(
        snmpEngine,
        CommunityData('public', mpModel=0),
        UdpTransportTarget(('demo.pysnmp.com', 162)),
        ContextData(),
        "trap",
        NotificationType(ObjectIdentity("1.3.6.1.6.3.1.1.5.2")).addVarBinds(
            ("1.3.6.1.6.3.1.1.4.3.0", "1.3.6.1.4.1.20408.4.1.1.2"),
            ("1.3.6.1.2.1.1.1.0", OctetString("my system")),
        ),
    )
    assert errorIndication is None
    assert errorStatus == 0
    assert errorIndex == 0
    assert varBinds == []
    snmpEngine.transportDispatcher.closeDispatcher()



import pytest
from pysnmp.hlapi.asyncio.slim import Slim
from pysnmp.smi.rfc1902 import ObjectIdentity, ObjectType

@pytest.mark.asyncio
async def test_v1_set():
    slim = Slim(1)
    errorIndication, errorStatus, errorIndex, varBinds = await slim.set(
        'public',
        'demo.pysnmp.com',
        161,
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysLocation", 0), "Shanghai")
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert len(varBinds) == 1
    assert varBinds[0][0].prettyPrint() == 'SNMPv2-MIB::sysLocation.0'
    assert varBinds[0][1].prettyPrint() == 'Shanghai'

    slim.close()


"""
Set scalar value
++++++++++++++++

Send a SNMP SET request
* with SNMPv2c with community name 'private'
* over IPv4/UDP
* to an Agent at 127.0.0.1:161
* for an OID in tuple form and an integer-typed value

This script performs similar to the following Net-SNMP command:

| $ snmpset -v2c -c private -ObentU 127.0.0.1:161 1.3.6.1.2.1.1.9.1.4.1 t 123

"""#
from pysnmp.entity import engine, config
from pysnmp.carrier.asyncore.dgram import udp
from pysnmp.entity.rfc3413 import cmdgen
from pysnmp.proto import rfc1902

# Create SNMP engine instance
snmpEngine = engine.SnmpEngine()

#
# SNMPv2c setup
#

# SecurityName <-> CommunityName mapping
config.addV1System(snmpEngine, "my-area", "private")

# Specify security settings per SecurityName (SNMPv1 - 0, SNMPv2c - 1)
config.addTargetParams(snmpEngine, "my-creds", "my-area", "noAuthNoPriv", 1)

#
# Setup transport endpoint and bind it with security settings yielding
# a target name
#

# UDP/IPv4
config.addTransport(
    snmpEngine, udp.domainName, udp.UdpSocketTransport().openClientMode()
)
config.addTargetAddr(
    snmpEngine, "my-router", udp.domainName, ("127.0.0.1", 161), "my-creds"
)


# Error/response receiver
# noinspection PyUnusedLocal,PyUnusedLocal,PyUnusedLocal
def cbFun(
    snmpEngine,
    sendRequestHandle,
    errorIndication,
    errorStatus,
    errorIndex,
    varBinds,
    cbCtx,
):
    if errorIndication:
        print(errorIndication)
    elif errorStatus:
        print(
            "{} at {}".format(
                errorStatus.prettyPrint(),
                errorIndex and varBinds[int(errorIndex) - 1][0] or "?",
            )
        )
    else:
        for oid, val in varBinds:
            print(f"{oid.prettyPrint()} = {val.prettyPrint()}")


# Prepare and send a request message
cmdgen.SetCommandGenerator().sendVarBinds(
    snmpEngine,
    "my-router",
    None,
    "",  # contextEngineId, contextName
    [((1, 3, 6, 1, 2, 1, 1, 9, 1, 4, 1), rfc1902.TimeTicks(123))],
    cbFun,
)

# Run I/O dispatcher which would send pending queries and process responses
snmpEngine.transportDispatcher.runDispatcher()



import pytest
from pysnmp.hlapi import *
from pysnmp.hlapi.asyncio.ntforg import sendNotification
from pysnmp.hlapi.asyncio.transport import UdpTransportTarget
from pysnmp.proto.errind import RequestTimedOut

@pytest.mark.asyncio
async def test_send_v3_inform_notification():
    snmpEngine = SnmpEngine()
    errorIndication, errorStatus, errorIndex, varBinds = await sendNotification(
        snmpEngine,
        UsmUserData('usr-md5-des', 'authkey1', 'privkey1'),
        UdpTransportTarget(('demo.pysnmp.com', 162)),
        ContextData(),
        'inform',
        NotificationType(
            ObjectIdentity('1.3.6.1.6.3.1.1.5.2')
        ).addVarBinds(
            ('1.3.6.1.2.1.1.1.0', OctetString('my system'))
        )
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert errorIndex == 0
    assert len(varBinds) == 3
    assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysUpTime.0"
    assert varBinds[1][0].prettyPrint() == "SNMPv2-MIB::snmpTrapOID.0"
    assert varBinds[2][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
    isinstance(varBinds[0][1], TimeTicks)
    isinstance(varBinds[1][1], ObjectIdentifier)
    isinstance(varBinds[2][1], OctetString)
    snmpEngine.transportDispatcher.closeDispatcher()



import asyncio
from pysnmp.hlapi.asyncio import *

async def run():
    snmpEngine = SnmpEngine()
    # Example of how you might update sysUpTime
    mibBuilder = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder
    sysUpTime, = mibBuilder.importSymbols('__SNMPv2-MIB', 'sysUpTime')
    sysUpTime.syntax = TimeTicks(12345)  # Set uptime to 12345

    errorIndication, errorStatus, errorIndex, varBinds = await sendNotification(
        snmpEngine,
        CommunityData('public', mpModel=0),
        UdpTransportTarget(('demo.pysnmp.com', 162)),
        ContextData(),
        "trap",
        NotificationType(ObjectIdentity('NET-SNMP-EXAMPLES-MIB', 'netSnmpExampleNotification')).addVarBinds(ObjectType(ObjectIdentity('NET-SNMP-EXAMPLES-MIB','netSnmpExampleHeartbeatRate'), 1))
    )

    if errorIndication:
        print(errorIndication)

    snmpEngine.transportDispatcher.closeDispatcher()


asyncio.run(run())


import pytest
from pysnmp.hlapi.asyncio import *

@pytest.mark.asyncio
async def test_v1_get():
    snmpEngine = SnmpEngine()
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        CommunityData('public', mpModel=0),
        UdpTransportTarget(('demo.pysnmp.com', 161)),
        ContextData(),
        ObjectType(ObjectIdentity('SNMPv2-MIB', 'sysDescr', 0))
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert len(varBinds) == 1
    assert varBinds[0][0].prettyPrint() == 'SNMPv2-MIB::sysDescr.0'
    assert isinstance(varBinds[0][1], OctetString)

    snmpEngine.transportDispatcher.closeDispatcher()


"""
INFORM, auth: MD5 privacy: DES
++++++++++++++++++++++++++++++
Send SNMP INFORM notification using the following options:
* SNMPv3
* with user 'usr-md5-des', auth: MD5, priv DES
* over IPv4/UDP
* send INFORM notification
* with TRAP ID 'warmStart' specified as a string OID
* include managed object information 1.3.6.1.2.1.1.5.0 = 'system name'
Functionally similar to:
| $ snmpinform -v3 -l authPriv -u usr-md5-des -A authkey1 -X privkey1 demo.pysnmp.com 12345 1.3.6.1.4.1.20408.4.1.1.2 1.3.6.1.2.1.1.1.0 s "my system"
"""#

import asyncio
from pysnmp.hlapi import *
from pysnmp.hlapi.asyncio.ntforg import sendNotification
from pysnmp.hlapi.asyncio.transport import UdpTransportTarget

async def run():
    errorIndication, errorStatus, errorIndex, varBinds = await sendNotification(
        SnmpEngine(),
        UsmUserData('usr-md5-des', 'authkey1', 'privkey1'),
        UdpTransportTarget(('demo.pysnmp.com', 162)),
        ContextData(),
        'inform',
        NotificationType(
            ObjectIdentity('1.3.6.1.6.3.1.1.5.2')
        ).addVarBinds(
            ObjectType(ObjectIdentity('1.3.6.1.2.1.1.5.0'), 'system name')
        ).loadMibs(
            'SNMPv2-MIB'
        )
    )

    if errorIndication:
        print(errorIndication)

    elif errorStatus:
        print('%s at %s' % (errorStatus.prettyPrint(),
                            errorIndex and varBinds[int(errorIndex) - 1][0] or '?'))

    else:
        for varBind in varBinds:
            print(' = '.join([x.prettyPrint() for x in varBind]))

asyncio.run(run())



import pytest
from pysnmp.hlapi.asyncio import *
from pysnmp.proto.errind import UnknownUserName

@pytest.mark.asyncio
async def test_usm_no_auth_no_priv():
    snmpEngine = SnmpEngine()
    authData = UsmUserData(
        "usr-none-none"
    )
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        authData,
        UdpTransportTarget(("demo.pysnmp.com", 161), retries=0),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert len(varBinds) == 1
    assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
    isinstance(varBinds[0][1], OctetString)

@pytest.mark.asyncio
async def test_usm_no_auth_no_priv_wrong_user():
    snmpEngine = SnmpEngine()
    authData = UsmUserData(
        "usr-none-none-not-exist"
    )
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        authData,
        UdpTransportTarget(("demo.pysnmp.com", 161), retries=0),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert isinstance(errorIndication, UnknownUserName)
    assert str(errorIndication) == 'Unknown USM user'
    snmpEngine.transportDispatcher.closeDispatcher()



"""
SNMPv1
++++++

Send SNMP GET request using the following options:

  * with SNMPv1, community 'public'
  * over IPv4/UDP
  * to an Agent at demo.pysnmp.com:161
  * for an instance of SNMPv2-MIB::sysDescr.0 MIB object
  * Based on asyncio I/O framework

Functionally similar to:

| $ snmpgetnext -v1 -c public demo.pysnmp.com SNMPv2-MIB::sysDescr.0

"""#
import asyncio
import pytest
from pysnmp.hlapi.asyncio.slim import Slim
from pysnmp.smi.rfc1902 import ObjectIdentity, ObjectType

@pytest.mark.asyncio
async def test_v1_next():
    slim = Slim()
    errorIndication, errorStatus, errorIndex, varBinds = await slim.next(
        'public',
        'demo.pysnmp.com',
        161,
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert errorIndex == 0
    assert len(varBinds) > 0

    slim.close()


"""
Custom ASN.1 MIB path
+++++++++++++++++++++

Send SNMP GET request using the following options:

* with SNMPv2c, community 'public'
* over IPv4/UDP
* to an Agent at demo.pysnmp.com:161
* for IF-MIB::ifInOctets.1 MIB object
* pass non-default ASN.1 MIB source to MIB compiler

Functionally similar to:

| $ snmpget -v2c -c public -M /usr/share/snmp demo.pysnmp.com IF-MIB::ifInOctets.1

"""#
import asyncio
from pysnmp.hlapi.asyncio import *

async def run():
    snmpEngine = SnmpEngine()
    errorIndication, errorStatus, errorIndex, varBinds = await getCmd(
        snmpEngine,
        CommunityData('public'),
        UdpTransportTarget(('demo.pysnmp.com', 161)),
        ContextData(),
        ObjectType(ObjectIdentity('IF-MIB', 'ifInOctets', 1).addAsn1MibSource('file:///usr/share/snmp',
                                                                                'https://mibs.pysnmp.com/asn1/@mib@'))
    )

    if errorIndication:
        print(errorIndication)
    elif errorStatus:
        print('%s at %s' % (errorStatus.prettyPrint(),
                            errorIndex and varBinds[int(errorIndex) - 1][0] or '?'))
    else:
        for varBind in varBinds:
            print(' = '.join([x.prettyPrint() for x in varBind]))

asyncio.run(run())



import pytest
from pysnmp.hlapi.asyncio.slim import Slim
from pysnmp.hlapi.asyncio import *

@pytest.mark.asyncio
async def test_v1_get():
    slim = Slim()
    errorIndication, errorStatus, errorIndex, varBinds = await slim.get(
        'public',
        'demo.pysnmp.com',
        161,
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
    )

    assert errorIndication is None
    assert errorStatus == 0
    assert len(varBinds) == 1
    assert varBinds[0][0].prettyPrint() == 'SNMPv2-MIB::sysDescr.0'
    assert isinstance(varBinds[0][1], OctetString)

    slim.close()


# http://www.python.org/dev/peps/pep-0396/
__version__ = '5.1.0'
# another variable is required to prevent semantic release from updating version in more than one place
main_version = __version__
# backward compatibility
# for beta versions, integer casting throws an exception, so string part must be cut off
if 'beta' in __version__:
    main_version = __version__.split('-beta')[0]
version = tuple(int(x) for x in main_version.split('.'))
majorVersionId = version[0]
